package sun.security.provider;

import java.io.BufferedInputStream;
import java.io.InputStream;
import java.security.cert.CertificateException;
import java.io.IOException;
import java.io.OutputStream;
import java.security.UnrecoverableEntryException;
import java.security.KeyStore;
import java.util.Enumeration;
import java.security.KeyStoreException;
import java.util.Date;
import java.security.cert.Certificate;
import java.security.UnrecoverableKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.Key;
import java.security.AccessController;
import java.security.Security;
import java.security.PrivilegedAction;
import sun.security.util.Debug;
import java.security.KeyStoreSpi;

class KeyStoreDelegator extends KeyStoreSpi
{
    private static final String KEYSTORE_TYPE_COMPAT = "keystore.type.compat";
    private static final Debug debug;
    private final String primaryType;
    private final String secondaryType;
    private final Class<? extends KeyStoreSpi> primaryKeyStore;
    private final Class<? extends KeyStoreSpi> secondaryKeyStore;
    private String type;
    private KeyStoreSpi keystore;
    private boolean compatModeEnabled;
    
    public KeyStoreDelegator(String primaryType, Class<? extends KeyStoreSpi> clazz, String s, Class<? extends KeyStoreSpi> secondaryKeyStore) {
        this.compatModeEnabled = true;
        this.compatModeEnabled = "true".equalsIgnoreCase((String)AccessController.doPrivileged(new PrivilegedAction<String>() {
            @Override
            public String run() {
                return Security.getProperty("keystore.type.compat");
            }
        }));
        if (this.compatModeEnabled) {
            this.primaryType = primaryType;
            this.secondaryType = s;
            this.primaryKeyStore = clazz;
            this.secondaryKeyStore = secondaryKeyStore;
        }
        else {
            this.primaryType = primaryType;
            this.secondaryType = null;
            this.primaryKeyStore = clazz;
            this.secondaryKeyStore = null;
            if (KeyStoreDelegator.debug != null) {
                KeyStoreDelegator.debug.println("WARNING: compatibility mode disabled for " + primaryType + " and " + s + " keystore types");
            }
        }
    }
    
    @Override
    public Key engineGetKey(String s, char[] array) throws NoSuchAlgorithmException, UnrecoverableKeyException {
        return this.keystore.engineGetKey(s, array);
    }
    
    @Override
    public Certificate[] engineGetCertificateChain(String s) {
        return this.keystore.engineGetCertificateChain(s);
    }
    
    @Override
    public Certificate engineGetCertificate(String s) {
        return this.keystore.engineGetCertificate(s);
    }
    
    @Override
    public Date engineGetCreationDate(String s) {
        return this.keystore.engineGetCreationDate(s);
    }
    
    @Override
    public void engineSetKeyEntry(String s, Key key, char[] array, Certificate[] array2) throws KeyStoreException {
        this.keystore.engineSetKeyEntry(s, key, array, array2);
    }
    
    @Override
    public void engineSetKeyEntry(String s, byte[] array, Certificate[] array2) throws KeyStoreException {
        this.keystore.engineSetKeyEntry(s, array, array2);
    }
    
    @Override
    public void engineSetCertificateEntry(String s, Certificate certificate) throws KeyStoreException {
        this.keystore.engineSetCertificateEntry(s, certificate);
    }
    
    @Override
    public void engineDeleteEntry(String s) throws KeyStoreException {
        this.keystore.engineDeleteEntry(s);
    }
    
    @Override
    public Enumeration<String> engineAliases() {
        return this.keystore.engineAliases();
    }
    
    @Override
    public boolean engineContainsAlias(String s) {
        return this.keystore.engineContainsAlias(s);
    }
    
    @Override
    public int engineSize() {
        return this.keystore.engineSize();
    }
    
    @Override
    public boolean engineIsKeyEntry(String s) {
        return this.keystore.engineIsKeyEntry(s);
    }
    
    @Override
    public boolean engineIsCertificateEntry(String s) {
        return this.keystore.engineIsCertificateEntry(s);
    }
    
    @Override
    public String engineGetCertificateAlias(Certificate certificate) {
        return this.keystore.engineGetCertificateAlias(certificate);
    }
    
    @Override
    public KeyStore.Entry engineGetEntry(String alias, KeyStore.ProtectionParameter protParam) throws KeyStoreException, NoSuchAlgorithmException, UnrecoverableEntryException {
        return this.keystore.engineGetEntry(alias, protParam);
    }
    
    @Override
    public void engineSetEntry(String alias, KeyStore.Entry entry, KeyStore.ProtectionParameter protParam) throws KeyStoreException {
        this.keystore.engineSetEntry(alias, entry, protParam);
    }
    
    @Override
    public boolean engineEntryInstanceOf(String alias, Class<? extends KeyStore.Entry> entryClass) {
        return this.keystore.engineEntryInstanceOf(alias, entryClass);
    }
    
    @Override
    public void engineStore(OutputStream outputStream, char[] array) throws IOException, NoSuchAlgorithmException, CertificateException {
        if (KeyStoreDelegator.debug != null) {
            KeyStoreDelegator.debug.println("Storing keystore in " + this.type + " format");
        }
        this.keystore.engineStore(outputStream, array);
    }
    
    @Override
    public void engineLoad(InputStream in, char[] array) throws IOException, NoSuchAlgorithmException, CertificateException {
        if (in != null && this.compatModeEnabled) {
            BufferedInputStream bufferedInputStream = new BufferedInputStream(in);
            bufferedInputStream.mark(Integer.MAX_VALUE);
            try {
                this.keystore = (KeyStoreSpi)this.primaryKeyStore.newInstance();
                this.type = this.primaryType;
                this.keystore.engineLoad(bufferedInputStream, array);
            }
            catch (Exception ex) {
                if (ex instanceof IOException && ex.getCause() instanceof UnrecoverableKeyException) {
                    throw (IOException)ex;
                }
                try {
                    this.keystore = (KeyStoreSpi)this.secondaryKeyStore.newInstance();
                    this.type = this.secondaryType;
                    bufferedInputStream.reset();
                    this.keystore.engineLoad(bufferedInputStream, array);
                    if (KeyStoreDelegator.debug != null) {
                        KeyStoreDelegator.debug.println("WARNING: switching from " + this.primaryType + " to " + this.secondaryType + " keystore file format has altered the keystore security level");
                    }
                }
                catch (InstantiationException | IllegalAccessException ex2) {}
                catch (IOException | NoSuchAlgorithmException | CertificateException ex3) {
                    if (ex3 instanceof IOException && ex3.getCause() instanceof UnrecoverableKeyException) {
                        throw (IOException)ex3;
                    }
                    if ((ex instanceof IOException)) {
                        throw (IOException)ex;
                    }
                    if ((ex instanceof CertificateException)) {
                        throw (CertificateException)ex;
                    }
                    if ((ex instanceof NoSuchAlgorithmException)) {
                        throw (NoSuchAlgorithmException)ex;
                    }
                }
            }
            if (KeyStoreDelegator.debug != null) {
                KeyStoreDelegator.debug.println("Loaded a keystore in " + this.type + " format");
                return;
            }
            return;
        }
        try {
            this.keystore = (KeyStoreSpi)this.primaryKeyStore.newInstance();
        }
        catch (InstantiationException | IllegalAccessException ex4) {}
        this.type = this.primaryType;
        if (KeyStoreDelegator.debug != null && in == null) {
            KeyStoreDelegator.debug.println("Creating a new keystore in " + this.type + " format");
        }
        this.keystore.engineLoad(in, array);
    }
    
    static {
        debug = Debug.getInstance("keystore");
    }
}
